初识 Linux 文件权限
================================================================================

在 Linux 中最基本的任务之一就是设置文件权限。理解如何实现是你进入 Linux 世界的第一步。如您所料，这一基本操作在类 UNIX 操作系统中大同小异。实际上，Linux 文件权限系统就直接取自于 UNIX 文件权限（甚至使用许多相同的工具）。

![](http://www.linux.com/images/stories/66866/files_a.png)

但不要以为理解文件权限需要长时间的学习。事实上会很简单，让我们一起来看看你需要了解哪些内容以及如何使用它们。

###基础概念

你要明白的第一件事是文件权限可以用来干什么。当你设置一个分组的权限时发生了什么？让我们将其展开来说，这个概念就真的简单多了。那到底什么是权限，什么是分组呢？

你可以设置的3种权限：

- 读 — 允许该分组读文件（用`r`表示）
- 写 — 允许该分组写文件（用`w`表示）
- 执行 — 允许该分组执行（运行）文件（用`x`表示）

为了更好地解释这如何应用于一个分组，例如，你允许一个分组可以读写一个文件，但不能执行。或者，你可以允许一个分组读和执行一个文件，但不能写。甚至你可以允许一个分组有读、写、执行全部的权限，也可以删除全部权限来去除该组的权限。

现在，什么是分组呢，有以下4个：

- user — 文件实际的拥有者
- group — 用户所在的用户组
- others — 用户组外的其他用户
- all — 所有用户

大多数情况，你只会对前3组进行操作，`all` 这一组只是作为快捷方式（稍后我会解释）。

到目前为止很简单，对吧？接下来我们将深入一层。

如果你打开一个终端并运行命令 `ls -l`，你将会看到逐行列出当前工作目录下所有的文件和文件夹的列表（如图1）.

你会留意到最左边那列是像是  `-rw-rw-r--` 这样的。

实际上这列表应该这样看：

> rw- rw- r--

正如你所见，列表将其分为如下3部分：

- rw-
- rw-
- r--

权限和组的顺序都很重要，顺序总是：

- 所属者 所属组 其他人 — 分组
- 读 写 执行 — 权限

在我们上面示例的权限列表中，所属者拥有读/写权限，所属组拥有读/写权限，其他人用户仅拥有读权限。这些分组中赋予执行权限的话，就用一个 x 表示。

### 等效数值

接下来我们让它更复杂一些,每个权限都可以用一个数字表示。这些数字是:

- 读 — 4
- 写 — 2
- 执行— 1

数值代替不是一个一个的替换，你不能像这样：

> -42-42-4--

你该把每个分组的数值相加，给用户读和写权限，你该用 4 + 2 得到 6。给用户组相同的权限，也是使用相同的数值。假如你只想给其他用户读的权限，那就设置它为4。现在用数值表示为：

> 664

如果你想给一个文件664权限，你可以使用 `chmod` 命令，如：

	chmod 664 FILENAME

FILENAME 处为文件名。

### 更改权限

既然你已经理解了文件权限，那是时候学习如何更改这些权限了。就是使用 `chmod` 命令来实现。第一步你要知道你能否更改文件权限，你必须是文件的所有者或者有权限编辑文件（或者通过 `su` 或 `sudo` 得到权限）。正因为这样，你不能随意切换目录和更改文件权限。

继续用我们的例子 (`-rw-rw-r--`)。假设这个文件（命名为 script.sh）实际是个shell脚本，需要被执行，但是你只想让自己有权限执行这个脚本。这个时候，你可能会想：“我需要是文件的权限如 `-rwx-rw-r--`”。为了设置 `x` 权限位，你可以这样使用 `chmod` 命令：

	chmod u+x script.sh

这时候，列表中显示的应该是 -rwx-rw-r-- 。

如果你想同时让用户及其所属组同时拥有执行权限，命令应该这样：

	chmod ug+x script.sh

明白这是怎么工作的了吗？下面我们让它更有趣些。不管什么原因，你不小心给了所有分组对文件的执行权限（列表中是这样的 `-rwx-rwx-r-x`）。

如果你想去除其他用户的执行权限，只需运行命令：

	chmod o-x script.sh

如果你想完全删除文件的可执行权限，你可以用两种方法：

	chmod ugo-x script.sh

或者

	chmod a-x script.sh

以上就是所有内容，能使操作更有效率。我希望能避免哪些可能会导致一些问题的操作（例如你不小心对 script.sh 使用 `a-rwx` 这样的 `chmod` 命令）。

### 目录权限

你也可以对一个目录执行 `chmod` 命令。当你作为用户创建一个新的目录，通常新建目录具有这样的权限：

> drwxrwxr-x

注：开头的 `d` 表示这是一个目录。

正如你所见，用户及其所在组都对文件夹具有操作权限，但这并不意味着在这文件夹中出创建的文件也具有与其相同的权限（创建的文件使用默认系统的权限 `-rw-rw-r--`）。但如果你想在新文件夹中创建文件，并且移除用户组的写权限，你不用切换到该目录下并对所有文件使用 `chmod` 命令。你可以用加上参数 R（意味着递归）的 `chmod` 命令，同时更改该文件夹及其目录下所有的文件的权限。

现在，假设有一文件夹 TEST，里面有一些脚本，所有这些（包括 TEST 文件夹）拥有权限 `-rwxrwxr-x`。如果你想移除用户组的写权限，你可以运行命令：

	chmod -R g-w TEST

运行命令 `ls -l`，你讲看到列出的 TEST 文件夹的权限信息是 `drwxr-xr-x`。用户组被去除了写权限（其目录下的所有文件也如此）。

### 总结

现在，你应该对基本的 Linux 文件权限有了深入的理解。对于更高级的东西学起来会很轻松，像 setgid、setuid 和 ACL 这些。没有良好的基础，你很快就会混淆不清概念的。

Linux 文件权限从早期到现在没有太大变化，而且很可能以后也不会变化。

------------------------------------------------------------------------------

via: http://www.linux.com/learn/tutorials/885268-getting-to-know-linux-file-permissions

作者：[Jack Wallen][a]
译者：[ynmlml](https://github.com/ynmlml)
校对：[wxy](https://github.com/wxy)

本文由 [LCTT](https://github.com/LCTT/TranslateProject) 原创翻译，[Linux中国](https://linux.cn/) 荣誉推出

[a]:http://www.linux.com/community/forums/person/93
