区块链进化简史：为什么开源是其核心所在
======

> 从比特币到下一代区块链。

![](https://opensource.com/sites/default/files/styles/image-full-size/public/lead-images/block-quilt-chain.png?itok=mECoDbrc)

当开源项目开发下一个新版本时，用后缀 “-ng” 表示 “下一代”的情况并不鲜见。幸运的是，到目前为止，快速演进的区块链成功地避开了这个命名陷阱。但是在这个开源生态系统的演进过程中，改变是不断发生的，而好的创意以典型的开源方式在许多不同的项目中被采用、交融和演进。

在本文中，我将审视不同代次的区块链，并且看一看在处理这个生态系统遇到的问题时出现什么创意。当然，任何对生态系统进行分类的尝试都有其局限性的 —— 和不同意见者的 —— 但是这也将为混乱的区块链项目提供了一个粗略的指南。

### 始作俑者：比特币

第一代的区块链起源于 <ruby>[比特币][1]<rt>Bitcoin</rt></ruby> 区块链，这是以去中心化、点对点加密货币为基础的<ruby>总帐<rt>ledger</rt></ruby>，它从 [Slashdot][2] 网站上的杂谈变成了一个主流话题。

这个区块链是一个分布式总帐，它对所有用户的<ruby>交易<rt>transaction</rt></ruby>保持跟踪，以避免他们<ruby>双重支付<rt>double-spending</rt></ruby>（双花）货币（在历史上，这个任务是委托给第三方—— 银行 ——来做的）。为防范攻击者在系统上捣乱，总帐被复制到每个参与到比特币网络的计算机上，并且每次只允许一台计算机去更新总帐。为决定哪台计算机能够获得更新总帐的权力，系统安排在比特币网络上的计算机之间每 10 分钟进行一场竞赛，这将消耗它们的（许多）能源才能参与竞赛。赢家将获得将前 10 分钟发生的交易写入到总帐（区块链中的“区块”）的权力，并且为赢家写入区块链的工作给予一些比特币奖励。这种方式被称为<ruby>工作量证明<rt>proof of work</rt></ruby>（PoW）共识机制。

这就是区块链最有趣的地方。比特币以[开源项目][3]的方式发布于 2009 年 1 月 。在 2010 年，由于意识到这些元素中的许多是可以调整的，围绕比特币聚集起了一个社区 —— [bitcointalk 论坛][4]，来开始各种实验。

起初，看到的比特币区块链是一个分布式数据库的形式， [Namecoin][5] 项目出现后，建议去保存任意数据到它的事务数据库中。如果区块链能够记录金钱的转移，那么它也应该能够记录其它资产的转移，比如域名。这正是 Namecoin 的主要使用场景，它上线于 2011 年 4 月 —— 也就是比特币出现两年后。

Namecoin 调整的地方是区块链的内容，<ruby>[莱特币][6]<rt>Litecoin</rt></ruby> 调整的是两个技术部分：一是将两个区块的时间间隔从 10 分钟减少到 2.5 分钟，二是改变了竞赛方式（用 [scrypt][7] 来替换了 SHA-256 安全哈希算法）。这是能够做到的，因为比特币是以开源软件的方式来发布的，而莱特币本质上与比特币在其它部分是完全相同的。莱特币是修改了比特币共识机制的第一个分叉，这也为其它的更多“币”铺平了道路。

沿着这条道路，基于比特币代码库的各种变种越来越多。其中一些扩展了比特币的用途，比如 [Zerocash][8] 协议，它专注于提供交易的匿名性和可替换性，但它最终分拆为它自己的货币 —— [Zcash][9]。

虽然 Zcash 带来了它自己的创新，使用了最近被称为“<ruby>零知识证明<rt>zero-knowledge proof</rt></ruby>”的加密技术，但它维持着与大多数主要的比特币代码库的兼容性，这意味着它能够从上游的比特币创新中获益。

另外的项目 —— [CryptoNote][10]，它萌芽于相同的社区，但是并没有使用相同的代码，它以比特币为背景来构建的，但又与之不同。它发布于 2012 年 12 月，由于它的出现，导致了几种加密货币的诞生，最著名的 <ruby>[门罗币][11]<rt>Monero</rt></ruby> （2014）就是其中之一。门罗币与 Zcash 使用了不同的方法，但解决了相同的问题：隐私性和可替换性。

就像在开源世界中经常出现的案例一样，做同样的工作有不止一个的工具可用。

### 下一代：“Blockchain-ng”

但是，到目前为止，所有的这些变体只是改进加密货币或者扩展它们去支持其它类型的事务。因此，这就引出了第二代区块链。

一旦社区开始去修改区块链的用法和调整技术部分时，对于一些想去扩展和重新思考它们未来的人来说，这种调整花费不了多长时间的。比特币的长期追随者 —— [Vitalik Buterin][12] 在 2013 年底建议，区域链的事务应该能够表示一个状态机的状态变化，将区域链视为能够运行应用程序（“<ruby>智能合约<rt>smart contract</rt></ruby>”）的分布式计算机。这个项目 —— <ruby>[以太坊][13]<rt>Ethereum</rt></ruby>，上线于 2015 年 4 月。它在运行分布式应用程序方面取得了巨大的成功，它的一些非常流行的分布式应用程序（<ruby>[加密猫][14]<rt>CryptoKitties</rt></ruby>）甚至导致以太坊区块链变慢。

这证明了目前的区块链存在一个很大的局限性：速度和容量。（速度通常用每秒事务数来测量，简称 TPS）有几个提议都建议去解决这个速度问题，从<ruby>分片<rt>sharding</rt></ruby>到<ruby>侧链<rt>sidechain</rt></ruby>，以及一个被称为“<ruby>第二层<rt>second-layer</rt></ruby>”的解决方案。这里需要更多的创新。

随着“智能合约”这个词开始流行起来，并且用已经被证实仍然很慢的技术去运行它们，那么就需要实现其它的思路：<ruby>许可区块链<rt>Permissioned blockchain</rt></ruby>。到目前为止，我们所介绍的所有区块链网络有两个没有明说的特征：一是它们是公开的（任何人都可以看到它们的功能），二是它们不需要许可（任何人都可以加入它们）。这两个部分是运行一个分布式的、非基于第三方的货币应该具有的和必需具有的条件。

随着区块链被认为出现与加密货币越来越明显的分离趋势，开始去考虑一些隐私、许可场景是很有意义的。一个有业务关系但不需要彼此完全信任的财团类型的参与者，能够从这些区块链类型中获益 —— 比如，物流链上的参与者，定期进行双边结算或者使用一个清算中心的金融、保险、或医疗保健机构。

一旦你将设置从“任何人都可以加入”变为“仅邀请者方可加入”，进一步对区块链构建区块的方式进行改变和调整将变得可能，那么对一些人来说，结果将变得非常有趣。

首先，设计用来保护网络不受恶意或者垃圾参与者的影响的工作量证明（PoW）可以被替换为更简单的和更少资源消耗的一些东西，比如，基于 [Raft][15] 的共识协议。在更高级别的安全性和更快的速度之间进行权衡，采用更简单的共识算法。对于更多群体来说这样更理想，因为他们可以用基于加密技术的担保来取代其它的基于法律关系的担保，例如为避免由于竞争而产生的大量能源消耗，而工作量证明就是这种情况。另外一个创新的地方是，使用 <ruby>[股权证明][16]<rt>Proof of Stake</rt></ruby>（PoS），它是公共网络共识机制的一个重量级的竞争者。它将可能像许可链网络一样找到它自己的实现方式。

有几个项目可以让创建许可区块链变得更简单，包括 [Quorum][17] （以太坊的一个分叉）和 [Hyperledger][18] 的 [Fabric][19] 和 [Sawtooth][20]，这是基于新代码的两个开源项目。

许可区块链可以避免公共的、非许可方式的区块链中某些错综复杂的问题，但是它自己也存在一些问题。正确地管理参与者是其中的一个问题：谁可以加入？如何辨别他们？如何将他们从网络上移除？网络上的一个实体是否去管理一个中央公共密钥基础设施（PKI）?

### 区块链的开放本质

到目前为止的所有案例中，有一件事情是很明确的：使用一个区块链的目标是去提升网络中的参与者和它产生的数据的信任水平，理想情况下，不需要做进一步的工作即可足以使用它。

只有为这个网络提供动力的软件是自由和开源的，才能达到这种信任水平。即便是一个正确的、专用的、分布式区块链，它的本质仍然是运行着相同的第三方代码的私有代理的集合。从本质上来说，区块链的源代码必须是开源的，但仅是开源还不够。随着生态系统持续成长，这既是最低限度的担保也是进一步创新的源头。

最后，值得一提的是，虽然区块链的开放本质被认为是创新和变化的源头，它也被认为是一种治理形式：代码治理，用户期望运行的任何一个特定版本，都应该包含他们认为的整个网络应该包含的功能和方法。在这方面，需要说明的一点是，一些区块链的开放本质正在“变味”。但是这一问题正在解决。

### 第三和第四代：治理

接下来，我正在考虑第三代和第四代区块链：区块链将内置治理工具，并且项目将去解决棘手的大量不同区块链之间互连互通的问题，以便于它们之间可以交换信息和价值。

---
关于作者

axel simon： 长期的自由及开源软件爱好者，就职于 Red Hat ，关注安全和区块链技术，以及分布式系统和协议。致力于保护互联网及其成就（知识分享、信息访问、去中心化和网络中立）。

--------------------------------------------------------------------------------

via: https://opensource.com/article/18/6/blockchain-guide-next-generation

作者：[Axel Simon][a]
选题：[lujun9972](https://github.com/lujun9972)
译者：[qhwdw](https://github.com/qhwdw)
校对：[wxy](https://github.com/wxy)

本文由 [LCTT](https://github.com/LCTT/TranslateProject) 原创编译，[Linux中国](https://linux.cn/) 荣誉推出

[a]:https://opensource.com/users/axel
[1]:https://bitcoin.org
[2]:https://slashdot.org/
[3]:https://github.com/bitcoin/bitcoin
[4]:https://bitcointalk.org/
[5]:https://www.namecoin.org/
[6]:https://litecoin.org/
[7]:https://en.wikipedia.org/wiki/Scrypt
[8]:http://zerocash-project.org/index
[9]:https://z.cash
[10]:https://cryptonote.org/
[11]:https://en.wikipedia.org/wiki/Monero_(cryptocurrency)
[12]:https://en.wikipedia.org/wiki/Vitalik_Buterin
[13]:https://ethereum.org
[14]:http://cryptokitties.co/
[15]:https://en.wikipedia.org/wiki/Raft_(computer_science)
[16]:https://www.investopedia.com/terms/p/proof-stake-pos.asp
[17]:https://www.jpmorgan.com/global/Quorum
[18]:https://hyperledger.org/
[19]:https://www.hyperledger.org/projects/fabric
[20]:https://www.hyperledger.org/projects/sawtooth
