How to write a really great resume that actually gets you hired
============================================================
 

![](https://cdn-images-1.medium.com/max/2000/1*k7HRLZAsuINP9vIs2BIh1g.png)

This is a data-driven guide to writing a resume that actually gets you hired. I’ve spent the past four years analyzing which resume advice works regardless of experience, role, or industry. The tactics laid out below are the result of what I’ve learned. They helped me land offers at Google, Microsoft, and Twitter and have helped my students systematically land jobs at Amazon, Apple, Google, Microsoft, Facebook, and more.

### Writing Resumes Sucks.

It’s a vicious cycle.

We start by sifting through dozens of articles by career “gurus,” forced to compare conflicting advice and make our own decisions on what to follow.

The first article says “one page MAX” while the second says “take two or three and include all of your experience.”

The next says “write a quick summary highlighting your personality and experience” while another says “summaries are a waste of space.”

You scrape together your best effort and hit “Submit,” sending your resume into the ether. When you don’t hear back, you wonder what went wrong:

 _“Was it the single page or the lack of a summary? Honestly, who gives a s**t at this point. I’m sick of sending out 10 resumes every day and hearing nothing but crickets.”_ 


![](https://cdn-images-1.medium.com/max/1000/1*_zQqAjBhB1R4fz55InrrIw.jpeg)
How it feels to try and get your resume read in today’s world.

Writing resumes sucks but it’s not your fault.

The real reason it’s so tough to write a resume is because most of the advice out there hasn’t been proven against the actual end goal of getting a job. If you don’t know what consistently works, you can’t lay out a system to get there.

It’s easy to say “one page works best” when you’ve seen it happen a few times. But how does it hold up when we look at 100 resumes across different industries, experience levels, and job titles?

That’s what this article aims to answer.

Over the past four years, I’ve personally applied to hundreds of companies and coached hundreds of people through the job search process. This has given me a huge opportunity to measure, analyze, and test the effectiveness of different resume strategies at scale.

This article is going to walk through everything I’ve learned about resumes over the past 4 years, including:

*   Mistakes that more than 95% of people make, causing their resumes to get tossed immediately

*   Three things that consistently appear in the resumes of highly effective job searchers (who go on to land jobs at the world’s best companies)

*   A quick hack that will help you stand out from the competition and instantly build relationships with whomever is reading your resume (increasing your chances of hearing back and getting hired)

*   The exact resume template that got me interviews and offers at Google, Microsoft, Twitter, Uber, and more

Before we get to the unconventional strategies that will help set you apart, we need to make sure our foundational bases are covered. That starts with understanding the mistakes most job seekers make so we can make our resume bulletproof.

### Resume Mistakes That 95% Of People Make

Most resumes that come through an online portal or across a recruiter’s desk are tossed out because they violate a simple rule.

When recruiters scan a resume, the first thing they look for is mistakes. Your resume could be fantastic, but if you violate a rule like using an unprofessional email address or improper grammar, it’s going to get tossed out.

Our goal is to fully understand the triggers that cause recruiters/ATS systems to make the snap decisions on who stays and who goes.

In order to get inside the heads of these decision makers, I collected data from dozens of recruiters and hiring mangers across industries. These people have several hundred years of hiring experience under their belts and they’ve reviewed 100,000+ resumes across industries.

They broke down the five most common mistakes that cause them to cut resumes from the pile:


![](https://cdn-images-1.medium.com/max/1000/1*5Zbr3HFeKSjvPGZdq_LCKA.png)

### The Five Most Common Resume Mistakes (According To Recruiters & Hiring Managers)

Issue #1: Sloppiness (typos, spelling errors, & grammatical mistakes). Close to 60% of resumes have some sort of typo or grammatical issue.

Solution: Have your resume reviewed by three separate sources — spell checking software, a friend, and a professional. Spell check should be covered if you’re using Microsoft Word or Google Docs to create your resume.

A friend or family member can cover the second base, but make sure you trust them with reviewing the whole thing. You can always include an obvious mistake to see if they catch it.

Finally, you can hire a professional editor on [Upwork][1]. It shouldn’t take them more than 15–20 minutes to review so it’s worth paying a bit more for someone with high ratings and lots of hours logged.

Issue #2: Summaries are too long and formal. Many resumes include summaries that consist of paragraphs explaining why they are a “driven, results oriented team player.” When hiring managers see a block of text at the top of the resume, you can bet they aren’t going to read the whole thing. If they do give it a shot and read something similar to the sentence above, they’re going to give up on the spot.

Solution: Summaries are highly effective, but they should be in bullet form and showcase your most relevant experience for the role. For example, if I’m applying for a new business sales role my first bullet might read “Responsible for driving $11M of new business in 2018, achieved 168% attainment (#1 on my team).”

Issue #3: Too many buzz words. Remember our driven team player from the last paragraph? Phrasing like that makes hiring managers cringe because your attempt to stand out actually makes you sound like everyone else.

Solution: Instead of using buzzwords, write naturally, use bullets, and include quantitative results whenever possible. Would you rather hire a salesperson who “is responsible for driving new business across the healthcare vertical to help companies achieve their goals” or “drove $15M of new business last quarter, including the largest deal in company history”? Skip the buzzwords and focus on results.

Issue #4: Having a resume that is more than one page. The average employer spends six seconds reviewing your resume — if it’s more than one page, it probably isn’t going to be read. When asked, recruiters from Google and Barclay’s both said multiple page resumes “are the bane of their existence.”

Solution: Increase your margins, decrease your font, and cut down your experience to highlight the most relevant pieces for the role. It may seem impossible but it’s worth the effort. When you’re dealing with recruiters who see hundreds of resumes every day, you want to make their lives as easy as possible.

### More Common Mistakes & Facts (Backed By Industry Research)

In addition to personal feedback, I combed through dozens of recruitment survey results to fill any gaps my contacts might have missed. Here are a few more items you may want to consider when writing your resume:

*   The average interviewer spends 6 seconds scanning your resume

*   The majority of interviewers have not looked at your resume until
     you walk into the room

*   76% of resumes are discarded for an unprofessional email address

*   Resumes with a photo have an 88% rejection rate

*   58% of resumes have typos

*   Applicant tracking software typically eliminates 75% of resumes due to a lack of keywords and phrases being present

Now that you know every mistake you need to avoid, the first item on your to-do list is to comb through your current resume and make sure it doesn’t violate anything mentioned above.

Once you have a clean resume, you can start to focus on more advanced tactics that will really make you stand out. There are a few unique elements you can use to push your application over the edge and finally get your dream company to notice you.


![](https://cdn-images-1.medium.com/max/1000/1*KthhefFO33-8tm0kBEPbig.jpeg)

### The 3 Elements Of A Resume That Will Get You Hired

My analysis showed that highly effective resumes typically include three specific elements: quantitative results, a simple design, and a quirky interests section. This section breaks down all three elements and shows you how to maximize their impact.

### Quantitative Results

Most resumes lack them.

Which is a shame because my data shows that they make the biggest difference between resumes that land interviews and resumes that end up in the trash.

Here’s an example from a recent resume that was emailed to me:

> Experience

> + Identified gaps in policies and processes and made recommendations for solutions at the department and institution level

> + Streamlined processes to increase efficiency and enhance quality

> + Directly supervised three managers and indirectly managed up to 15 staff on multiple projects

> + Oversaw execution of in-house advertising strategy

> + Implemented comprehensive social media plan

As an employer, that tells me absolutely nothing about what to expect if I hire this person.

They executed an in-house marketing strategy. Did it work? How did they measure it? What was the ROI?

They also also identified gaps in processes and recommended solutions. What was the result? Did they save time and operating expenses? Did it streamline a process resulting in more output?

Finally, they managed a team of three supervisors and 15 staffers. How did that team do? Was it better than the other teams at the company? What results did they get and how did those improve under this person’s management?

See what I’m getting at here?

These types of bullets talk about daily activities, but companies don’t care about what you do every day. They care about results. By including measurable metrics and achievements in your resume, you’re showcasing the value that the employer can expect to get if they hire you.

Let’s take a look at revised versions of those same bullets:

> Experience

> + Managed a team of 20 that consistently outperformed other departments in lead generation, deal size, and overall satisfaction (based on our culture survey)

> + Executed in-house marketing strategy that resulted in a 15% increase in monthly leads along with a 5% drop in the cost per lead

> + Implemented targeted social media campaign across Instagram & Pintrest, which drove an additional 50,000 monthly website visits and generated 750 qualified leads in 3 months

If you were in the hiring manager’s shoes, which resume would you choose?

That’s the power of including quantitative results.

### Simple, Aesthetic Design That Hooks The Reader

These days, it’s easy to get carried away with our mission to “stand out.” I’ve seen resume overhauls from graphic designers, video resumes, and even resumes [hidden in a box of donuts.][2]

While those can work in very specific situations, we want to aim for a strategy that consistently gets results. The format I saw the most success with was a black and white Word template with sections in this order:

*   Summary

*   Interests

*   Experience

*   Education

*   Volunteer Work (if you have it)

This template is effective because it’s familiar and easy for the reader to digest.

As I mentioned earlier, hiring managers scan resumes for an average of 6 seconds. If your resume is in an unfamiliar format, those 6 seconds won’t be very comfortable for the hiring manager. Our brains prefer things we can easily recognize. You want to make sure that a hiring manager can actually catch a glimpse of who you are during their quick scan of your resume.

If we’re not relying on design, this hook needs to come from the  _Summary_ section at the top of your resume.

This section should be done in bullets (not paragraph form) and it should contain 3–4 highlights of the most relevant experience you have for the role. For example, if I was applying for a New Business Sales position, my summary could look like this:

> Summary

> Drove quarterly average of $11M in new business with a quota attainment of 128% (#1 on my team)

> Received award for largest sales deal of the year

> Developed and trained sales team on new lead generation process that increased total leads by 17% in 3 months, resulting in 4 new deals worth $7M

Those bullets speak directly to the value I can add to the company if I was hired for the role.

### An “Interests” Section That’s Quirky, Unique, & Relatable

This is a little “hack” you can use to instantly build personal connections and positive associations with whomever is reading your resume.

Most resumes have a skills/interests section, but it’s usually parked at the bottom and offers little to no value. It’s time to change things up.

[Research shows][3] that people rely on emotions, not information, to make decisions. Big brands use this principle all the time — emotional responses to advertisements are more influential on a person’s intent to buy than the content of an ad.

You probably remember Apple’s famous “Get A Mac” campaign:


When it came to specs and performance, Macs didn’t blow every single PC out of the water. But these ads solidified who was “cool” and who wasn’t, which was worth a few extra bucks to a few million people.

By tugging at our need to feel “cool,” Apple’s campaign led to a [42% increase in market share][4] and a record sales year for Macbooks.

Now we’re going to take that same tactic and apply it to your resume.

If you can invoke an emotional response from your recruiter, you can influence the mental association they assign to you. This gives you a major competitive advantage.

Let’s start with a question — what could you talk about for hours?

It could be cryptocurrency, cooking, World War 2, World of Warcraft, or how Google’s bet on segmenting their company under the Alphabet is going to impact the technology sector over the next 5 years.

Did a topic (or two) pop into year head? Great.

Now think about what it would be like to have a conversation with someone who was just as passionate and knew just as much as you did on the topic. It’d be pretty awesome, right?  _Finally, _ someone who gets it!

That’s exactly the kind of emotional response we’re aiming to get from a hiring manager.

There are five “neutral” topics out there that people enjoy talking about:

1.  Food/Drink

2.  Sports

3.  College

4.  Hobbies

5.  Geography (travel, where people are from, etc.)

These topics are present in plenty of interest sections but we want to take them one step further.

Let’s say you had the best night of your life at the Full Moon Party in Thailand. Which of the following two options would you be more excited to read:

*   Traveling

*   Ko Pha Ngan beaches (where the full moon party is held)

Or, let’s say that you went to Duke (an ACC school) and still follow their basketball team. Which would you be more pumped about:

*   College Sports

*   ACC Basketball (Go Blue Devils!)

In both cases, the second answer would probably invoke a larger emotional response because it is tied directly to your experience.

I want you to think about your interests that fit into the five categories I mentioned above.

Now I want you to write a specific favorite associated with each category in parentheses next to your original list. For example, if you wrote travel you can add (ask me about the time I was chased by an elephant in India) or (specifically meditation in a Tibetan monastery).

Here is the [exact set of interests][5] I used on my resume when I interviewed at Google, Microsoft, and Twitter:

 _ABC Kitchen’s Atmosphere, Stumptown Coffee (primarily cold brew), Michael Lewis (Liar’s Poker), Fishing (especially fly), Foods That Are Vehicles For Hot Sauce, ACC Sports (Go Deacs!) & The New York Giants_ 


![](https://cdn-images-1.medium.com/max/1000/1*ONxtGr_xUYmz4_Xe66aeng.jpeg)

If you want to cheat here, my experience shows that anything about hot sauce is an instant conversation starter.

### The Proven Plug & Play Resume Template

Now that we have our strategies down, it’s time to apply these tactics to a real resume. Our goal is to write something that increases your chances of hearing back from companies, enhances your relationships with hiring managers, and ultimately helps you score the job offer.

The example below is the exact resume that I used to land interviews and offers at Microsoft, Google, and Twitter. I was targeting roles in Account Management and Sales, so this sample is tailored towards those positions. We’ll break down each section below:


![](https://cdn-images-1.medium.com/max/1000/1*B2RQ89ue2dGymRdwMY2lBA.png)

First, I want you to notice how clean this is. Each section is clearly labeled and separated and flows nicely from top to bottom.

My summary speaks directly to the value I’ve created in the past around company culture and its bottom line:

*   I consistently exceeded expectations

*   I started my own business in the space (and saw real results)

*   I’m a team player who prioritizes culture

I purposefully include my Interests section right below my Summary. If my hiring manager’s six second scan focused on the summary, I know they’ll be interested. Those bullets cover all the subconscious criteria for qualification in sales. They’re going to be curious to read more in my Experience section.

By sandwiching my Interests in the middle, I’m upping their visibility and increasing the chance of creating that personal connection.

You never know — the person reading my resume may also be a hot sauce connoisseur and I don’t want that to be overlooked because my interests were sitting at the bottom.

Next, my Experience section aims to flesh out the points made in my Summary. I mentioned exceeding my quota up top, so I included two specific initiatives that led to that attainment, including measurable results:

*   A partnership leveraging display advertising to drive users to a gamified experience. The campaign resulted in over 3000 acquisitions and laid the groundwork for the 2nd largest deal in company history.

*   A partnership with a top tier agency aimed at increasing conversions for a client by improving user experience and upgrading tracking during a company-wide website overhaul (the client has ~20 brand sites). Our efforts over 6 months resulted in a contract extension worth 316% more than their original deal.

Finally, I included my education at the very bottom starting with the most relevant coursework.

Download My Resume Templates For Free

You can download a copy of the resume sample above as well as a plug and play template here:

Austin’s Resume: [Click To Download][6]

Plug & Play Resume Template: [Click To Download][7]

### Bonus Tip: An Unconventional Resume “Hack” To Help You Beat Applicant Tracking Software

If you’re not already familiar, Applicant Tracking Systems are pieces of software that companies use to help “automate” the hiring process.

After you hit submit on your online application, the ATS software scans your resume looking for specific keywords and phrases (if you want more details, [this article][8] does a good job of explaining ATS).

If the language in your resume matches up, the software sees it as a good fit for the role and will pass it on to the recruiter. However, even if you’re highly qualified for the role but you don’t use the right wording, your resume can end up sitting in a black hole.

I’m going to teach you a little hack to help improve your chances of beating the system and getting your resume in the hands of a human:

Step 1: Highlight and select the entire job description page and copy it to your clipboard.

Step 2: Head over to [WordClouds.com][9] and click on the “Word List” button at the top. Towards the top of the pop up box, you should see a link for Paste/Type Text. Go ahead and click that.

Step 3: Now paste the entire job description into the box, then hit “Apply.”

WordClouds is going to spit out an image that showcases every word in the job description. The larger words are the ones that appear most frequently (and the ones you want to make sure to include when writing your resume). Here’s an example for a data a science role:


![](https://cdn-images-1.medium.com/max/1000/1*O7VO1C9nhC9LZct7vexTbA.png)

You can also get a quantitative view by clicking “Word List” again after creating your cloud. That will show you the number of times each word appeared in the job description:

9 data

6 models

4 experience

4 learning

3 Experience

3 develop

3 team

2 Qualifications

2 statistics

2 techniques

2 libraries

2 preferred

2 research

2 business

When writing your resume, your goal is to include those words in the same proportions as the job description.

It’s not a guaranteed way to beat the online application process, but it will definitely help improve your chances of getting your foot in the door!

* * *

### Want The Inside Info On Landing A Dream Job Without Connections, Without “Experience,” & Without Applying Online?

[Click here to get the 5 free strategies that my students have used to land jobs at Google, Microsoft, Amazon, and more without applying online.][10]

 _Originally published at _  [_cultivatedculture.com_][11]  _._

--------------------------------------------------------------------------------

作者简介:

I help people land jobs they love and salaries they deserve at CultivatedCulture.com

----------

via: https://medium.freecodecamp.org/how-to-write-a-really-great-resume-that-actually-gets-you-hired-e18533cd8d17

作者：[Austin Belcak ][a]
译者：[译者ID](https://github.com/译者ID)
校对：[校对者ID](https://github.com/校对者ID)

本文由 [LCTT](https://github.com/LCTT/TranslateProject) 原创编译，[Linux中国](https://linux.cn/) 荣誉推出

[a]:https://medium.freecodecamp.org/@austin.belcak
[1]:http://www.upwork.com/
[2]:https://www.thrillist.com/news/nation/this-guy-hides-his-resume-in-boxes-of-donuts-to-score-job-interviews
[3]:https://www.psychologytoday.com/blog/inside-the-consumer-mind/201302/how-emotions-influence-what-we-buy
[4]:https://www.businesswire.com/news/home/20070608005253/en/Apple-Mac-Named-Successful-Marketing-Campaign-2007
[5]:http://cultivatedculture.com/resume-skills-section/
[6]:https://drive.google.com/file/d/182gN6Kt1kBCo1LgMjtsGHOQW2lzATpZr/view?usp=sharing
[7]:https://drive.google.com/open?id=0B3WIcEDrxeYYdXFPVlcyQlJIbWc
[8]:https://www.jobscan.co/blog/8-things-you-need-to-know-about-applicant-tracking-systems/
[9]:https://www.wordclouds.com/
[10]:https://cultivatedculture.com/dreamjob/
[11]:https://cultivatedculture.com/write-a-resume/