4 open source alternatives to Microsoft Access
======
Build simple business applications and keep track of your data with these worthy open source alternatives.
![](https://opensource.com/sites/default/files/styles/image-full-size/public/lead-images/bus_cloud_database.png?itok=lhhU42fg)

When small businesses, community organizations, and similar-sized groups realize they need software to manage their data, they think first of Microsoft Access. That may be the right choice if you're already paying for a Microsoft Office subscription or don't care that it's proprietary. But it's far from your only option—whether you prefer to use open source alternatives from a philosophical standpoint or you don't have the big budget for a Microsoft Office subscription—there are several open source database applications that are worthy alternatives to proprietary software like Microsoft Access or Apple FileMaker.

If that sounds like you, here are four open source database tools for your consideration.

### LibreOffice Base

![](https://opensource.com/sites/default/files/uploads/libreoffice-base.png)
In case it's not obvious from its name, [Base][1] is part of the [LibreOffice][2] productivity suite, which includes Writer (word processing), Calc (spreadsheet), Impress (presentations), Draw (graphics), Charts (chart creation), and Math (formulas). As such, Base integrates with the other LibreOffice applications, much like Access does with the Microsoft Office suite. This means you can import and export data from Base into the suite's other applications to create financial reports, mail merges, charts, and more.

Base includes drivers that natively support multi-user database engines, including the open source MySQL, MariaDB, and PostgreSQL; Access; and other JDBC and ODBC-compliant databases. Built-in wizards and table definitions make it easy for new users to quickly get started building tables, writing queries, and creating forms and reports (such as invoices, sales reports, and customer lists). To learn more, consult the comprehensive [user manual][3] and dive into the [user forums][4]. If you're still stuck, you can find a [certified][5] support professional to help you out.

Installers are available for Linux, MacOS, Windows, and Android. LibreOffice is available under the [Mozilla Public License v2][6]; if you'd like to join the large contributor community and help improve the software, visit the [Get Involved][7] section of LibreOffice's website.

### DB Browser for SQLite

![](https://opensource.com/sites/default/files/uploads/sqlitebrowser.png)

[DB Browser for SQLite][8] enables users to create and use SQLite database files without having to know complex SQL commands. This, plus its spreadsheet-like interface and pre-built wizards, make it a great option for new database users to get going without much background knowledge.

Although the application has gone through several name changes—from the original Arca Database Browser to the SQLite Database Browser and finally to the current name (in 2014, to avoid confusion with SQLite), it's stayed true to its goal of being easy for users to operate.

Its wizards enable users to easily create and modify database files, tables, indexes, records, etc.; import and export data to common file formats; create and issue queries and searches; and more. Installers are available for Windows, MacOS, and a variety of Linux versions, and its [wiki on GitHub][9] offers a wealth of information for users and developers.

DB Browser for SQLite is [bi-licensed][10] under the Mozilla Public License Version 2 and the GNU General Public License Version 3 or later, and you can download the source code from the project's website.

### Kexi

![](https://opensource.com/sites/default/files/uploads/kexi-3.0-table-view.png)
As the database application in the [Calligra Suite][11] productivity software for the KDE desktop, [Kexi][12] integrates with the other applications in the suite, including Words (word processing), Sheets (spreadsheet), Stage (presentations), and Plan (project management).

As a full member of the [KDE][13] project, Kexi is purpose-built for KDE Plasma, but it's not limited to KDE users: Linux, BSD, and Unix users running GNOME can run the database, as can MacOS and Windows users.

Kexi's website says its development was "motivated by the lack of rapid application development ([RAD][14]) tools for database systems that are sufficiently powerful, inexpensive, open standards driven, and portable across many operating systems and hardware platforms." It has all the standard features you'd expect: designing databases, storing data, doing queries, processing data, and so forth.

Kexi is available under the [LGPL][15] open source license and you can download its [source code][16] from its development wiki. If you'd like to learn more, take a look at its [user handbook][17], [forums][18], and [userbase wiki][17].

### nuBuilder Forte

![](https://opensource.com/sites/default/files/uploads/screenshot_from_2018-10-17_13-23-25.png)
[NuBuilder Forte][19] is designed to be as easy as possible for people to use. It's a browser-based tool for developing web-based database applications.

Its clean interface and low-code tools (including support for drag-and-drop) allow users to create and use a database quickly. As a fully web-based application, data is accessible anywhere from a browser. Everything is stored in MySQL and can be backed up in one database file.

It uses industry-standard coding languages—HTML, PHP, JavaScript, and SQL—making it easy for developers to get started also.

Help is available in [videos][20] and other [documentation][21] for topics including creating forms, doing searches, building reports, and more.

nuBuilder Forte is licensed under [GPLv3.0][22] and you can download it on [GitHub][23]. You can learn more by consulting the [nuBuilder Forum][24] or watching its [demo][25] video.

Do you have a favorite open source database tool for building simple projects with little or no coding skill required? If so, please share in the comments.

--------------------------------------------------------------------------------

via: https://opensource.com/alternatives/access

作者：[Opensource.com][a]
选题：[lujun9972][b]
译者：[译者ID](https://github.com/译者ID)
校对：[校对者ID](https://github.com/校对者ID)

本文由 [LCTT](https://github.com/LCTT/TranslateProject) 原创编译，[Linux中国](https://linux.cn/) 荣誉推出

[a]: https://opensource.com
[b]: https://github.com/lujun9972
[1]: https://www.libreoffice.org/discover/base/
[2]: https://www.libreoffice.org/
[3]: https://documentation.libreoffice.org/en/english-documentation/base/
[4]: http://document-foundation-mail-archive.969070.n3.nabble.com/Users-f1639498.html
[5]: https://www.libreoffice.org/get-help/professional-support/
[6]: https://www.libreoffice.org/download/license/
[7]: https://www.libreoffice.org/community/get-involved/
[8]: http://sqlitebrowser.org/
[9]: https://github.com/sqlitebrowser/sqlitebrowser/wiki
[10]: https://github.com/sqlitebrowser/sqlitebrowser/blob/master/LICENSE
[11]: https://www.calligra.org/
[12]: https://www.calligra.org/kexi/
[13]: https://www.kde.org/
[14]: http://en.wikipedia.org/wiki/Rapid_application_development
[15]: http://kexi-project.org/wiki/wikiview/index.php@KexiLicense.html
[16]: http://kexi-project.org/wiki/wikiview/index.php@Download.html
[17]: https://userbase.kde.org/Kexi/Handbook
[18]: http://forum.kde.org/kexi
[19]: https://www.nubuilder.com/
[20]: https://www.nubuilder.com/videos
[21]: https://www.nubuilder.com/wiki
[22]: https://github.com/nuSoftware/nuBuilder4/blob/master/LICENSE.txt
[23]: https://github.com/nuSoftware/nuBuilder4
[24]: https://forums.nubuilder.com/viewforum.php?f=18&sid=7036bccdc08ba0da73181bc72cd63c62
[25]: https://www.youtube.com/watch?v=tdh9ILCUAco&feature=youtu.be
